@@ -5,12 +5,13 @@ set -o errexit  # Exit the script with error if any of the commands fail
 # Supported/used environment variables:
 #   LINK_STATIC                Whether to statically link to libmongoc
 #   BUILD_SAMPLE_WITH_CMAKE    Link program w/ CMake. Default: use pkg-config.
+#   BUILD_SAMPLE_WITH_CMAKE_DEPRECATED  If BUILD_SAMPLE_WITH_CMAKE is set, then use deprecated CMake scripts instead.
 #   ENABLE_SSL                 Set -DENABLE_SSL
 #   ENABLE_SNAPPY              Set -DENABLE_SNAPPY
 #   CMAKE                      Path to cmake executable.
 
 
-echo "LINK_STATIC=$LINK_STATIC BUILD_SAMPLE_WITH_CMAKE=$BUILD_SAMPLE_WITH_CMAKE"
+echo "LINK_STATIC=$LINK_STATIC BUILD_SAMPLE_WITH_CMAKE=$BUILD_SAMPLE_WITH_CMAKE BUILD_SAMPLE_WITH_CMAKE_DEPRECATED=$BUILD_SAMPLE_WITH_CMAKE_DEPRECATED"
 
 DIR=$(dirname $0)
 . $DIR/find-cmake.sh
@@ -72,8 +73,16 @@ else
   STATIC_CMAKE_OPTION="-DENABLE_STATIC=OFF -DENABLE_TESTS=OFF"
 fi
 
+# Since zstd inconsitently installed on macos-1014
+# Remove this check in CDRIVER-3483.
+if [ "darwin" = "$OS" ]; then
+   ZSTD="OFF"
+else
+   ZSTD="AUTO"
+fi
 
-$CMAKE -DCMAKE_INSTALL_PREFIX=$INSTALL_DIR -DCMAKE_PREFIX_PATH=$INSTALL_DIR/lib/cmake $SSL_CMAKE_OPTION $SNAPPY_CMAKE_OPTION $STATIC_CMAKE_OPTION -DENABLE_BSON=ON .
+
+$CMAKE -DCMAKE_INSTALL_PREFIX=$INSTALL_DIR -DCMAKE_PREFIX_PATH=$INSTALL_DIR/lib/cmake $SSL_CMAKE_OPTION $SNAPPY_CMAKE_OPTION $STATIC_CMAKE_OPTION -DENABLE_BSON=ON -DENABLE_CLIENT_SIDE_ENCRYPTION=OFF -DENABLE_ZSTD=$ZSTD .
 make
 make install
 
@@ -114,17 +123,23 @@ if test ! -f $INSTALL_DIR/lib/pkgconfig/libmongoc-1.0.pc; then
 else
   echo "libmongoc-1.0.pc check ok"
 fi
-if test ! -f $INSTALL_DIR/lib/cmake/libmongoc-1.0/libmongoc-1.0-config.cmake; then
-  echo "libmongoc-1.0-config.cmake missing!"
+if test ! -f $INSTALL_DIR/lib/cmake/mongoc-1.0/mongoc-1.0-config.cmake; then
+  echo "mongoc-1.0-config.cmake missing!"
   exit 1
 else
-  echo "libmongoc-1.0-config.cmake check ok"
+  echo "mongoc-1.0-config.cmake check ok"
 fi
-if test ! -f $INSTALL_DIR/lib/cmake/libmongoc-1.0/libmongoc-1.0-config-version.cmake; then
-  echo "libmongoc-1.0-config-version.cmake missing!"
+if test ! -f $INSTALL_DIR/lib/cmake/mongoc-1.0/mongoc-1.0-config-version.cmake; then
+  echo "mongoc-1.0-config-version.cmake missing!"
   exit 1
 else
-  echo "libmongoc-1.0-config-version.cmake check ok"
+  echo "mongoc-1.0-config-version.cmake check ok"
+fi
+if test ! -f $INSTALL_DIR/lib/cmake/mongoc-1.0/mongoc-targets.cmake; then
+  echo "mongoc-targets.cmake missing!"
+  exit 1
+else
+  echo "mongoc-targets.cmake check ok"
 fi
 
 
@@ -141,18 +156,6 @@ if [ "$LINK_STATIC" ]; then
   else
     echo "libmongoc-static-1.0.pc check ok"
   fi
-  if test ! -f $INSTALL_DIR/lib/cmake/libmongoc-static-1.0/libmongoc-static-1.0-config.cmake; then
-    echo "libmongoc-static-1.0-config.cmake missing!"
-    exit 1
-  else
-    echo "libmongoc-static-1.0-config.cmake check ok"
-  fi
-  if test ! -f $INSTALL_DIR/lib/cmake/libmongoc-static-1.0/libmongoc-static-1.0-config-version.cmake; then
-    echo "libmongoc-static-1.0-config-version.cmake missing!"
-    exit 1
-  else
-    echo "libmongoc-static-1.0-config-version.cmake check ok"
-  fi
 else
   if test -f $INSTALL_DIR/lib/libmongoc-static-1.0.a; then
     echo "libmongoc-static-1.0.a shouldn't have been installed"
@@ -166,14 +169,6 @@ else
     echo "libmongoc-static-1.0.pc shouldn't have been installed"
     exit 1
   fi
-  if test -f $INSTALL_DIR/lib/cmake/libmongoc-static-1.0/libmongoc-static-1.0-config.cmake; then
-    echo "libmongoc-static-1.0-config.cmake shouldn't have been installed"
-    exit 1
-  fi
-  if test -f $INSTALL_DIR/lib/cmake/libmongoc-static-1.0/libmongoc-static-1.0-config-version.cmake; then
-    echo "libmongoc-static-1.0-config-version.cmake shouldn't have been installed"
-    exit 1
-  fi
 fi
 
 if [ "$OS" = "darwin" ]; then
@@ -194,7 +189,11 @@ set -o xtrace
 
 if [ "$BUILD_SAMPLE_WITH_CMAKE" ]; then
   # Test our CMake package config file with CMake's find_package command.
-  EXAMPLE_DIR=$SRCROOT/src/libmongoc/examples/cmake/find_package
+  if [ "$BUILD_SAMPLE_WITH_CMAKE_DEPRECATED" ]; then
+    EXAMPLE_DIR=$SRCROOT/src/libmongoc/examples/cmake-deprecated/find_package
+  else
+    EXAMPLE_DIR=$SRCROOT/src/libmongoc/examples/cmake/find_package
+  fi
 
   if [ "$LINK_STATIC" ]; then
     EXAMPLE_DIR="${EXAMPLE_DIR}_static"
